package main

import (
	"crypto/ecdh"
	"flag"
	"fmt"
	"os"

	"drat/internal/drat"
	"drat/internal/mu"
)

const usage = `Usage: bob [options] BOB_HOST

The "Bob" (initial-receiver) party of a double-ratchet communication.

positional arguments:
  BOB_HOST 
    The "[host]:port" string for Bob's address.  Host can be either an
    IP address, a host name (e.g., 'localhost') or omitted altogether.
    
options:
  -secret SECRET_KEY
    The file that contains the 32-byte shared secret that serves as the
    initial root key.

    Default: secret.key

  -pub PUBLIC_KEY
    A file with Bob's raw public X25519 key (that is, the file is the raw key
    bytes, rather than a DER or PEM-encoding of these bytes).

    Default: bob-pub.key

  -priv PRIVATE_KEY
    A file with Bob's raw private X25519 key.

    Default: bob-priv.key

  -help
    Display this usage statement and exit.

examples:
$ ./bob -secret assets/secrek.key -pub assets/bob-pub.key -priv assets/bob-priv.key 127.0.0.1:12345
`

type Options struct {
	// positional
	hostport string
	// options
	pubKeyFile    string
	pubKey        *ecdh.PublicKey // derived
	privKeyFile   string
	privKey       *ecdh.PrivateKey // derived
	secretKeyFile string
	secretKey     []byte // derived
}

func printUsage() {
	fmt.Fprintf(os.Stderr, "%s", usage)
}

func parseOptions() *Options {
	var err error
	options := Options{}

	flag.Usage = printUsage
	flag.StringVar(&options.pubKeyFile, "pub", "bob-pub.key", "")
	flag.StringVar(&options.privKeyFile, "priv", "bob-priv.key", "")
	flag.StringVar(&options.secretKeyFile, "secret", "secret.key", "")
	flag.Parse()

	if flag.NArg() != 1 {
		mu.Die("expected one positional argument but got %d", flag.NArg())
	}

	options.hostport = flag.Arg(0)

	options.secretKey, err = drat.LoadSecretKey(options.secretKeyFile)
	if err != nil {
		mu.Die("error loading secret key from %s: %v", options.secretKeyFile, err)
	}

	options.pubKey, err = drat.LoadPubDH(options.pubKeyFile)
	if err != nil {
		mu.Die("error loading Bob's public key from %s: %v", options.pubKeyFile, err)
	}

	options.privKey, err = drat.LoadPrivDH(options.privKeyFile)
	if err != nil {
		mu.Die("error loading Bob's private key from %s: %v", options.privKeyFile, err)
	}

	return &options
}

func main() {
	options := parseOptions()

	// TODO: delete the mu.UNUSED line and implement the rest of the program
	mu.UNUSED(options)
}
